/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;
import java.net.*;

/**
 * A class that produces a ImageLabel. No mouse or
 * action events are defined, so it is most similar to the
 * Label Component.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class ImageLabel extends Canvas {

	/** The actual Image drawn on the canvas. Don't specify this directly;
	 *  pass a string, URL, or image to the ImageLabel constructor.
	 */
	protected Image image;

	/** A String corresponding to the URL of the image you will get
	 *  if you call the constructor with no arguments.
	 */
	public static String defaultImageString
		= "http://java.sun.com/lib/images/logo.java.color-transp.55x60.gif";
	
	/** The URL of the image. But sometimes we will use an existing image
	 *  object (e.g. made by createImage) for which this info will not be
	 *  available, so a default string is used here.
	 */
	protected String imageString = "<Existing Image>";

	/** Amount of extra space at the left side of the image. */
	protected int marginL = 0;

	/** Amount of extra space above the image. */
	protected int marginT = 0;

	/** Amount of extra space at the right side of the image. */	
	protected int marginR = 0;

	/** Amount of extra space below the image. */	
	protected int marginB = 0;

	/** Amount of extra space around the image. This value is used for
	 * setting all margins together */	
	protected int margin = 0;
	
	/** The actual size of the image */
	protected Dimension imSize = null;

	/** If this value is true the image is stretched or contracted into
	 * the area of ImageLabel. Otherwise the image is painted 
	 * in the actual size of image without resizing.
	 */
	protected boolean stretch = true;

	private MediaTracker tracker;
	private static int lastTrackerID=0;
	private int currentTrackerID;

	protected boolean doneLoading = false;
	protected Container parentContainer;

	/** Create an ImageLabel with the default image.
	 * @see #defaultImageString
	 */
	public ImageLabel() {
		this(defaultImageString);
	}

	/** Create an ImageLabel using the image at URL specified by the string.
	 * @param imageURLString A String specifying the URL of the image.
	 */
	public ImageLabel(String imageURLString) {
		this(makeURL(imageURLString));
	}

	/** Create an ImageLabel using the image at URL specified.
	 * @param imageURL The URL of the image.
	 */
	public ImageLabel(URL imageURL) {
		this(getImage(imageURL));
		imageString = imageURL.toExternalForm();
	}

	/** Create an ImageLabel using the image specified. The other
	 *  constructors eventually call this one, but you may want to call
	 *  it directly if you already have an image (e.g. created via
	 *  createImage).
	 * @param image The image
	 */
	public ImageLabel(Image image) {
		this.image = image;
		tracker = new MediaTracker(this);
		currentTrackerID = lastTrackerID++;
		tracker.addImage(image, currentTrackerID);
		setDefaults();
	}

	private void setDefaults() {
		imSize = new Dimension(0,0);
		setBorderWidth(0);
	}

	/** Sets border size around the image
	 * @param borderWidth New value of margin
	 * @see #margin
	 * @see #getBorderWidth
	 * @see #setMargins
	 */
	public void setBorderWidth(int borderWidth) {
		margin = borderWidth;
		setMargins();
	}

	/** Sets marginL, marginT, marginR, marginB to the value of
	 * margin. It is called by setBorderWidth.
	 * @see #margin
	 * @see #marginL
	 * @see #marginR
	 * @see #marginT
	 * @see #marginB
	 * @see #setBorderWidth
	 */
	protected void setMargins() {
		marginL = margin;
		marginR = margin;
		marginT = margin;
		marginB = margin;
	}

	/** Gets border width around the image.
	 * @see #margin
	 * @see #setBorderWidth
	 */
	public int getBorderWidth() {
		return(margin);
	}

	/** Makes sure that the Image associated with the Canvas is done loading
	 *  before returning.
	 *  getImage spins off a separate thread to do the loading. Once you
	 *  get around to drawing the image, this will make sure it is loaded,
	 *  waiting if not. The user does not need to call this at all, but
	 *  if several ImageLabels are used on the same Comtainer, this can cause
	 *  several repated layouts, so users might want to explicitly call this
	 *  themselves before adding the ImageLabel to the Container. On the
	 *  other hand, postponing the waiting as long as possible is more
	 *  efficient, since loading can go on in the background.
	 *
	 * @param doLayout Determines if the Container should be re-layed out
	 *                 after you are finished waiting. <B>This should be
	 *                 true when called from user functions</B>, but
	 *                 is set to false when called from preferredSize
	 *                 to avoid an infinite loop. This is needed when
	 *                 using BorderLayout, which calls preferredSize
	 *                 <B>before</B> calling paint.
	 */
	public void waitForImage(boolean doLayout) {
		if (!doneLoading) {
			try { 
				tracker.waitForID(currentTrackerID);
			}
			catch (InterruptedException i) {}
			catch (Exception e) {
				System.out.println("Error loading " + imageString + ": "
				   + e.getMessage());
				e.printStackTrace();
			}
			if (tracker.isErrorID(0))
				new Throwable("Error loading image " + imageString).printStackTrace();
			doneLoading = true;
			Dimension d = size();
			imSize.width = image.getWidth(this);
			imSize.height = image.getHeight(this);

			parentContainer = getParent();
			if ((parentContainer != null) && doLayout) {
				setBackground(parentContainer.getBackground());
				parentContainer.layout();
			}
		}
	}

	public void paint(Graphics g) {
		if (!doneLoading)
			waitForImage(true);
		else {
			Dimension d = size();
			if (stretch)
				g.drawImage(image, marginL, marginT, d.width-marginL-marginR,
					d.height - marginT - marginB, this);
			else
				g.drawImage(image, marginL, marginT, imSize.width, 
					imSize.height, this);
			drawBorder(g);
		}
	}

	public Dimension preferredSize() {
		if (!doneLoading)
			waitForImage(false);
		return new Dimension(imSize.width+marginL+marginR, imSize.height+marginT+marginB);
	}

	public Dimension minimumSize() {
		if (!doneLoading)
			waitForImage(false);
		return new Dimension(imSize.width+marginL+marginR, imSize.height+marginT+marginB);
	}

	/** Draws border of the image. By default it does nothing. So ImageLabel
	 * has the default border color the same color as background color.
	 * It should be overwrite by the descendats to do something reasonable.
	 */
	protected void drawBorder(Graphics g) {
	}

	private static URL makeURL(String s) {
		URL u = null;
		try { u = new URL(s); }
		catch (MalformedURLException m) {
			System.out.println("Bad URL " + s + ": " + m);
			m.printStackTrace();
		}
		return(u);
	}

	private static Image getImage(URL url) {
		return(Toolkit.getDefaultToolkit().getImage(url));
	}

	/** Sets stretch mode of ImageLabel.
	 * @param aStretch New value of stretch variable.
	 * @see #stretch
	 * @see #getStretch
	 */
	public void setStretch(boolean aStretch) {
		stretch = aStretch;
		repaint();
	}
	
	/** Gets actual value of stretch variable.
	 * @see #stretch
	 * @see #setStretch
	 */
	public boolean getStretch() {
		return stretch;
	}
}
