/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;
import java.util.*;

/**
 * A class that produces a tab control component. It can be used
 * separately or in the PageControl.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class TabControl extends Canvas {

	/** When user change selected tab in TabControl, 
	 *  parent will get the event with this id. 
	 */
    public static final int TAB_CHANGE = 47421;

	/** Current style of TabControl. If it is true, TabControl
	 *  direction down otherwise up.
	 * @see #setDirection
	 * @see #getDirection
	 * @see #defaultDirection
	 */
    public boolean direction;

	/** Default style of TabControl. Currently true.
	 * @see #setDirection
	 * @see #getDirection
	 * @see #direction
	 */
	public static final boolean defaultDirection = true;

	/** The basic color of the 3D border. Left and top sides are 
	 *  lighter and right and bottom sides are darker then this 
	 *  color. By default it is set to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently lightGray.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
	public static final Color defaultBorderColor = new Color(192, 192, 192);

	/** Current width of the 3D border around the component
	 * @see #defaultBorderWidth
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */	
	protected int margin = 0;

	/** Default value of margin. Currently 1.
	 * @see #margin
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */
    public static final int defaultBorderWidth = 1;

	/** Guarantee minimum width of each tab.
	 * @see #setMinWidth
	 * @see #getMinWidth
	 * @see #defaultMinWidth
	 */
    protected int minWidth;
	
	/** Default value of guarantee minimum width of each tab.
	 * @see #setMinWidth
	 * @see #getMinWidth
	 * @see #minWidth
	 */
    public static final int defaultMinWidth = 50;

	/** Currently selected tab.
	 * @see #setTabIndex
	 * @see #getTabIndex
	 */
    protected int tabIndex;

    /** Vector of Strings with names of all tabs. */
	protected Vector tabs;

	/** Indent of labels each tab. */
    protected static final int indent = 4;

	/** Vector of sizes each tab. Depends on font size and the width
	 *  of the label.
	 */
    protected Vector sizes;

	/** First visible tab in case that not all tabs are together
	 *  visible. When the width of TabControl component is less than
	 *  needed minimum width at the right side will appear small
	 *  spin buttons for shifting visible tabs.
	 * @see #lastVisible
	 * @see #allVisible
	 */
    protected int firstVisible;

	/** Last visible tab in case that not all tabs are together
	 *  visible. When the width of TabControl component is less than
	 *  needed minimum width at the right side will appear small
	 *  spin buttons for shifting visible tabs.
	 * @see #firstVisible
	 * @see #allVisible
	 */
    protected int lastVisible;

	/** Flag if all tabs are visible.
	 * @see #firstVisible
	 * @see #lastVisible
	 */
    protected boolean allVisible;

	/** Size of small spin buttons for shifting of the visible
	 *  tabs.
	 * @see #allVisible
	 */
    protected static final int posButtonSize = 10;

	/** If propagate is set to true all events are propagate and don't
	 *  catch. This flag is used for xawt component during design time.
	 */
	public boolean propagate = false;

	/** Create new TabControl with all default values. */
    public TabControl() {
	    tabIndex = -1;
	    tabs = new Vector();
	    sizes = new Vector();
	    setDefaults();
    }

    private void setDefaults() {
        margin = defaultBorderWidth;
        marginColor = defaultBorderColor;
        minWidth = defaultMinWidth;
   	    direction = defaultDirection;
   	    firstVisible = 0;
   	    lastVisible = 0;
   	    allVisible = true;
    }

	/** Select new tab.
	 * @param index Index of new selected tab.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 * @see #tabIndex
	 * @see #getTabIndex
	 */
    public void setTabIndex(int index) throws ArrayIndexOutOfBoundsException {
	    if ((index < 0) || (index >= tabs.size()))
	        throw new ArrayIndexOutOfBoundsException();
	    tabIndex = index;
	    repaint();
    }

	/** Gets the index of currently selected tab.
	 * @return Index of selected tab.
	 * @see #tabIndex
	 * @see #setTabIndex
	 */
    public int getTabIndex() {
	    return tabIndex;
    }

	/** Gets count of the tabs.
	 * @return Count of tabs.
	 */
    public int getTabCount() {
        return tabs.size();
    }

	/** Gets the name of the tab with the specific index.
	 * @param index Index of tab.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 * @return Name of tab.
	 */
    public String getTabName(int index) throws ArrayIndexOutOfBoundsException {
	    return (String) tabs.elementAt(index);
    }

	/** Add new tab.
	 * @param newTab Name of new tab.
	 */
    public void addTab(String newTab) {
        int index = tabs.size();
        addTabAt(newTab, index);
    }

	/** Add new tab at specific position.
	 * @param newTab Name of new tab.
	 * @param index Position of new tab.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 */
    public void addTabAt(String newTab, int index) throws
            ArrayIndexOutOfBoundsException {
        if (tabIndex == -1)
            tabIndex = 0;
        tabs.insertElementAt(newTab, index);

		Font font = getFont();
		FontMetrics f = Toolkit.getDefaultToolkit().getFontMetrics(
				font==null ? new Font("Helvetica", Font.PLAIN, 12) : font);

        int w = f.stringWidth(newTab) + 2 * (indent + margin) + 1;
        if (w < minWidth)
            w = minWidth;
        sizes.insertElementAt(new Integer(w), index);
        checkWidth(true);
        repaint();
    }

	/** Removes the tab at the specific position.
	 * @param index Position of tab.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 */
    public void removeTabAt(int index) throws
            ArrayIndexOutOfBoundsException {
        tabs.removeElementAt(index);
        sizes.removeElementAt(index);
		if (tabs.size() == 0)
			tabIndex = -1;
        if ((tabIndex > index) && (tabIndex > 0))
            tabIndex--;
        firstVisible--;
        lastVisible--;
        checkWidth(true);
        repaint();
    }

	/** Remove all tabs.
	 * @see #removeTabAt
	 */
	public void removeAllTabs() {
        tabs.removeAllElements();
        sizes.removeAllElements();
	    tabIndex = -1;
   	    firstVisible = 0;
   	    lastVisible = 0;
   	    allVisible = true;
        repaint();
	}

	/** Sets the TabControl to be visible tab with the specific
	 *  index.
	 * @see #firstVisible
	 * @see #lastVisible
	 * @see #allVisible
	 */
	public void setVisible(int index) {
		if ((index < firstVisible) || (index > lastVisible)) {
			firstVisible = index;
			lastVisible = index;
	        checkWidth(true);
		}
	}

    private void checkWidth(boolean dir) {
        Dimension d = size();
        int w = countTotalSize();
        if (w <= d.width) {
            if (!allVisible)
                allVisible = true;
            else {
                firstVisible = 0;
                lastVisible = tabs.size() - 1;
            }
        }
        else {
            int wMax = d.width - (4 * margin + 2 * posButtonSize + 2 * indent);
            if (wMax < 0) {
                firstVisible = 0;
                lastVisible = 0;
                allVisible = false;
            }
            else {
                if (allVisible) {
                    allVisible = false;
                    if (countTotalSize(0, tabIndex) < wMax) {
                        firstVisible = 0;
                        for (lastVisible = tabIndex; lastVisible < sizes.size(); lastVisible++)
                            if (countTotalSize(0, lastVisible) > wMax) {
                                lastVisible--;
                                break;
                            }
                    }
                    else {
                        lastVisible = tabIndex;
                        for (firstVisible = 0; firstVisible <= lastVisible; firstVisible++)
                            if (countTotalSize(firstVisible, lastVisible) < wMax)
                                break;
                    }
                }
                else {
                    if (countTotalSize(firstVisible, lastVisible) < wMax) {
                        for (firstVisible = 0; firstVisible < lastVisible; firstVisible++)
                            if (countTotalSize(firstVisible, lastVisible) < wMax) {
                                break;
                            }
                        for (; lastVisible < sizes.size() - 1; lastVisible++)
                            if (countTotalSize(firstVisible, lastVisible) > wMax) {
                                lastVisible--;
                                break;
                            }
                    }
                    else { // dir = true (reduce from left), dir = false (reduce from right)
                        if (dir) {
                            for (; firstVisible < lastVisible; firstVisible++)
                                if (countTotalSize(firstVisible, lastVisible) < wMax)
                                    break;
                        }
                        else {
                            for (; lastVisible > firstVisible; lastVisible--)
                                if (countTotalSize(firstVisible, lastVisible) < wMax)
                                    break;
                        }
                    }
                }
            }
        }
    }

	/** Sets a new direction of TabControl. If it is true, 
	 *  TabControl direction down otherwise up.
	 * @param direct New direction.
	 * @see #direction
	 * @see #getDirection
	 */
    public void setDirection(boolean direct) {
        if (direction != direct) {
            direction = direct;
            repaint();
        }
    }

	/** Gets the current direction of TabControl. If it is true, 
	 *  TabControl direction down otherwise up.
	 * @return Current direction.
	 * @see #direction
	 * @see #setDirection
	 */
    public boolean getDirection() {
        return direction;
    }

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see #margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
        countSizes();
        checkWidth(true);
        repaint();
    }

	/** Gets border width around the component.
	 * @see #margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
        return(margin);
    }

	/** Sets the minimum width of each tab.
	 * @aMinWidth New value of minWidth variable.
	 * @see #minWidth
	 * @see #getMinWidth
	 */
    public void setMinWidth(int aMinWidth) {
        minWidth = aMinWidth;
        countSizes();
        checkWidth(true);
        repaint();
    }

	/** Gets the minimum width of each tab.
	 * @return Current value of minWidth variable.
	 * @see #minWidth
	 * @see #setMinWidth
	 */
    public int getMinWidth() {
        return(minWidth);
    }

	public synchronized void setFont(Font f) {
		super.setFont(f);
		countSizes();
        checkWidth(true);
        repaint();
	}

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
        repaint();
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return marginColor;
    }

    public void paint(Graphics g) {
		Dimension s = size();
		if (tabs.size() > 0) {
            int horizTrans = 2;
	        int start = 0;
			Font font = getFont();
			FontMetrics f = Toolkit.getDefaultToolkit().getFontMetrics(
					font==null ? new Font("Helvetica", Font.PLAIN, 12) : font);

	        int fontYPos;
	        if (s.height < f.getHeight())
	            fontYPos = s.height;
	        else
	            fontYPos = (s.height + f.getHeight()) / 2;

	        int focusStart = 0;
	        int focusWidth = 0;
			if (allVisible) {
				firstVisible = 0;
				lastVisible = sizes.size() - 1;
			}

	        for (int t = firstVisible; t <= lastVisible; t++) {
	            int top = 0;
	            int left = start;
	            int height = s.height - 1;
	            int width = ((Integer) sizes.elementAt(t)).intValue();
	            int fontYPosDelta = 0;

	            if (tabIndex == t) {
	                focusStart = start;
	                focusWidth = width - 1;
	                if (!direction) {
	                    fontYPosDelta -= horizTrans;
	                    top -= horizTrans;
	                }
	                else {
	                    height += margin;
	                }
	            }
	            else {
	                if (direction) {
	                    top += horizTrans;
	                    fontYPosDelta += horizTrans;
	                }
	            }
	      		g.setColor(getForeground());
	            String str = new String((String) tabs.elementAt(t));
	            int delta = f.stringWidth(str);
	            delta = ((width - delta - 2* (margin + indent)) / 2);
	            g.drawString(str, start + margin + indent + delta, fontYPos + fontYPosDelta);

	            start += width;

	            width -=1;
                g.setColor(Color.black);
                g.drawLine(left + width, top, left + width, top + height);

	            g.setColor(marginColor);
	            width -=1;
                if ((tabIndex == t) && (!direction)) {
                    top -= margin;
                    height += margin;
                }
	            for(int i = 0; i < margin; i++) {
	                g.draw3DRect(left, top, width, height, true);
	                left++;
	                top++;
	                width = width - 2;
	                height = height - 2;
	            }
	        }
            if (!allVisible) {
                int top = s.height - indent - 2 * margin - posButtonSize;
                int height = posButtonSize + 2 * margin;
                int width = height;
                int left = s.width - indent - 4 * margin - 2 * posButtonSize;
                int[] xP = new int[3];
                int[] yP = new int[3];
                int os = left + 2 * margin + posButtonSize;
                int difr = posButtonSize / 10;
                for (int q = 0; q <= 1; q++) {
                    boolean b = true;
                    switch (q) {
                        case 0: xP[0] = os - margin - posButtonSize + difr;
                                xP[1] = os - margin - difr;
                                xP[2] = xP[1];
                                yP[0] = top + margin + posButtonSize / 2;
                                yP[1] = top + margin + difr;
                                yP[2] = top + margin + posButtonSize - difr;
                                if (firstVisible == 0)
                                    b = false;
                                break;
                        case 1: for (int j = 0; j <= 2; j++)
                                    xP[j] = 2 * os - xP[j];
                                if (lastVisible == tabs.size() - 1)
                                    b = false;
                    }
                    if (b) {
                        g.setColor(marginColor);
                        for (int i = 0; i < margin; i++)
                            g.draw3DRect(left + i, top + i, width - 2*i, height - 2*i, true);
                        g.setColor(getForeground());
                        g.fillPolygon(xP, yP, 3);
                    }
                    left += 2 * margin + posButtonSize;
                }
            }
            if (direction) {
                g.setColor(marginColor);
                for (int i = 0; i < margin; i++) {
                    g.draw3DRect(0, s.height - i - 1, focusStart + margin, margin, true);
                    g.draw3DRect(focusStart + focusWidth - margin + i + 1, s.height - i - 1,
                        s.width - focusStart - focusWidth + margin, margin, true);
                }
                g.setColor(getBackground());
                g.fillRect(focusStart + margin, s.height - margin - 1, 1, margin + 1);
            }
            else {
                g.setColor(marginColor);
                for (int i = 0; i < margin; i++) {
                    if (tabIndex != firstVisible)
                        g.draw3DRect(margin - i - 1, -1, focusStart, 1 + i, true);
                    g.draw3DRect(focusStart + focusWidth - margin - 1, -1,
                        s.width - focusStart - focusWidth + margin, 1 + i, true);
                }
                g.setColor(getBackground());
                g.fillRect(focusStart + focusWidth - margin - 1 , 0, 1, margin);
                g.setColor(Color.black);
                g.drawLine(focusStart, s.height - horizTrans, focusStart + focusWidth, s.height - horizTrans);
                g.drawLine(focusStart + focusWidth, margin, s.width, margin);
                if (tabIndex != firstVisible)
                    g.drawLine(0, margin, focusStart - 1, margin);
            }
        }
		else {
			g.setColor(getBackground());
			g.fillRect(0, 0, s.width, s.height);
			g.setColor(getForeground());
			g.drawRect(0,0, s.width - 1, s.height - 1);
		}
    }

    public Dimension minimumSize() {
        int w = countMaxSize();
		Font font = getFont();
		FontMetrics f = Toolkit.getDefaultToolkit().getFontMetrics(
				font==null ? new Font("Helvetica", Font.PLAIN, 12) : font);

        if (sizes.size() > 1) {
            w += 2 * indent + 2 * posButtonSize + 4 * margin;
            w = Math.min(w, countTotalSize());
        }
        int h = 2 * margin + f.getHeight() * 3 / 2;
        return new Dimension(w, h);
    }

    public Dimension preferredSize() {
        int w = countTotalSize();
		if (w == 0)
			w = 2 * margin;
		Font font = getFont();
		FontMetrics f = Toolkit.getDefaultToolkit().getFontMetrics(
				font==null ? new Font("Helvetica", Font.PLAIN, 12) : font);

        int h = 2 * margin + f.getHeight() * 3 / 2;
        return new Dimension(w, h);
    }

    public void resize(int width, int height) {
        super.resize(width, height);
        int a = firstVisible;
        int b = lastVisible;
        checkWidth(false);
        if ((a != firstVisible) || (b != lastVisible))
            repaint();
    }

    public void resize(Dimension d) {
		resize(d.width, d.height);
    }


    public void reshape(int x, int y, int width, int height) {
        super.reshape(x, y, width, height);
        int a = firstVisible;
        int b = lastVisible;
        checkWidth(false);
        if ((a != firstVisible) || (b != lastVisible))
            repaint();
    }

    private void countSizes() {
        int n = tabs.size();
        sizes.removeAllElements();
        if (n > 0) {
            FontMetrics f = getFontMetrics(getFont());
            for (int i = 0; i < n; i++) {
                int w = f.stringWidth((String) tabs.elementAt(i)) + 2 * (indent + margin) + 1;
                if (w < minWidth)
                    w = minWidth;
                sizes.addElement(new Integer(w));
            }
        }
    }

    private int countTotalSize(int from, int till) {
        int result = 0;
        for (int i = from; i <= till; i++)
            result += ((Integer) sizes.elementAt(i)).intValue();
        return result;
    }

    private int countTotalSize() {
        return countTotalSize(0, sizes.size() - 1);
    }

    private int countMaxSize() {
        int result = 0;
        for (int i = 0; i < sizes.size(); i++)
            result = Math.max(result, ((Integer) sizes.elementAt(i)).intValue());
        return result;
    }

    public boolean mouseDown(Event event, int x, int y) {
		if (propagate)
			return false;
        int sum = 0;
        int i;
        int old = tabIndex;
        for (i = firstVisible; i <= lastVisible; i++) {
            sum += ((Integer) sizes.elementAt(i)).intValue();
            if (x < sum)
                break;
        }
        if ((x < sum) && (tabIndex != i)){
            tabIndex = i;
            repaint();
            event.id = TAB_CHANGE;
            event.arg = new Integer(old);
            return false;
        }
        else {
            if (!allVisible) {
                Dimension s = size();
                int tmp = s.height - indent;
                if ((y >= tmp - 2 * margin - posButtonSize) && (y < tmp)) {
                    tmp = s.width - indent - 4 * margin - 2 * posButtonSize;
                    if ((x >= tmp) && (x < tmp + 2 * margin + posButtonSize) &&
                        (firstVisible > 0)) {
                            firstVisible--;
                            checkWidth(false);
                            repaint();
                    }
                    if ((x >= tmp + 2 * margin + posButtonSize) &&
                        (x < tmp + 4 * margin + 2 * posButtonSize) && (lastVisible < tabs.size() - 1)) {
                            lastVisible++;
                            checkWidth(true);
                            repaint();
                    }
                }
            }
            return true;
        }
    }
}


