/** ErrorDescription class
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.compiler;

import java.io.IOException;

import xelfi.top.*;
import xelfi.editor.*;
import xelfi.xedit.*;



  /** Holds info about one error line in one class - line number, column and the comment.
   */
public class ErrorDescription implements OutputCallback
{
    /** All errors in one class */
  protected ErrorList parent;

    /** line number (first line = 1)*/
  int line;

    /** column (first = 1)*/
  int col;

    /** Comment of the error. */
  String comment;

    /** index of the line with text with this error. */
  int lineInOutput;

    /** Creates a new err. description for specified line. The other data are filled later. 
      @param l line number. 
    */
  public ErrorDescription(int l)
  {
    parent = null;
    line = l;
    col = 1;    // default - beginning of a line
    comment = null;
    lineInOutput = -l;
  }

    /** Error description can contain a handle to the corresponding text to output window. 
      This sets it. 
      @param l handle to output line (used in output window).
      */
  public void setOutputLine(int l)
  {
    lineInOutput = l;
  }

    /** Error description can contain a column. This assigneds it to this. 
      @param c the column
     */
  public void setColumn(int c)
  {
    col = c;
  }

    /** Error description can contain a comment. This  assigneds it to this.
      @param com the comment
     */
  public void setComment(String com)
  {
    comment = com;
  }

    /** Counts the position of this error. 
      @return the position
     */
  Position getPosition()
  {
    Snapshot snap = TopLevel.getCurrentSnapshot();
    Source s = null;
    try
    {
      s = new Source(parent.getClassName());
    }
    catch(IOException e) { System.err.println("Error in errors colorer: cannot open Source"); }
    return snap.getPosition(s, line - 1).column(col - 1);
  }

    /** Scrolles editor window and conditionally the Output window into this error.
      @param ed editor window for the class source
      @param alsoOutput whether to scroll the Output as well.
    */
  public void scroll(JavaEditorFrame ed, boolean alsoOutput)
  {
    Position p = getPosition();
    ed.getTextEditor().setCursorPos(p);
    Block.line(p).redraw();
    if(comment != null) ed.setStatusLine(comment);
    if(alsoOutput && lineInOutput != -1) TopLevel.getOutput().select(lineInOutput);
  }

    /** OutputCallback implementation - called when error is selected from the Output window.*/
  public void selection(int index)
  {
    parent.setHighlightedError(this);

      // scroll one Editor and force the error line to redraw
    JavaEditorFrame ed = TopLevel.findEditor(parent.getClassName());
    if(ed != null) scroll(ed, false);
  }

    /** OutputCallback implementation - called when error is selected from the Output window.*/
  public void action(int index)
  {
    parent.setHighlightedError(this);
    scroll(TopLevel.showEditor(parent.getClassName(), true), false);
  }
    /** OutputCallback implementation - called when error is cleared from the Output window.*/
  public void clear(int index)
  {
    lineInOutput = -1;
    ErrorDatabase.removeErrors(parent.getClassName());
  }

  void setParent(ErrorList p)
  {
    parent = p;
  }
}

