/**
* Xelfi
*/

package xelfi.design.forms;

import java.util.Vector;
import java.io.DataInput;
import java.io.DataOutput;
import xelfi.design.components.XelfiDesignLayout;
import xelfi.util.EnhProperties;
import xelfi.util.Debug;

/**
* This class manages all installed layouts for whole Xelfi
* (thus all its methods are static).
* @version 		1.04, 23 Feb 1997 
* @finished
* @author 		Ian Formanek
*/

public class XFInstalledLayouts {
	private static Vector layoutNames = new Vector();
	private static Vector classNames = new Vector();
	private static boolean isDefault = false;

	/**
	* return code : layout installed OK
	*/
	public static int IL_OK = 0;
	/**
	* return code : IllegalAccesException occured (probably the layout has no constructor)
	*/
	public static int IL_ILLEGAL_ACCESS = -1;
	/**
	* return code : InstantiationException occured (the layout class is abstract)
	*/
	public static int IL_INSTANTIATION = -2;
	/**
	* return code : ClassNotFoundException occured (the layout class could not have been found)
	*/
	public static int IL_CLASS_NOT_FOUND = -3;
	/**
	* return code : Other problem occured
	*/
	public static int IL_OTHER = -4;
	
	/**
	* This method returns array of names of all currently installed layouts. 
	* The Names are names of the layout classes not of the design layout classes 
	* (e.g. FlowLayout, not XFlowLayout).
	* This method is intended primarily for ComponentInspector.
	* @return array of names of currently installed layouts
	*/
	public static String[] getAllLayoutNames() {
		int items = layoutNames.size();
		String list[] = new String[items];
		for (int i=0; i<items; i++)
			list[i] = new String((String)layoutNames.elementAt(i));
		return list;
	}

	/**
	* This method installs given layout.
	* Given class name must be complete (i.e. with packages) name to design layout class 
	* (e.g. xelfi.design.forms.layouts.XFlowLayout).
	* @param className	complete name of design layout class to install
	* @return one of IL_??? return codes
	* @see #uninstallLayout
	* @see #IL_OK
	* @see #IL_ILLEGAL_ACCESS
	* @see #IL_INSTANTIATION
	* @see #IL_CLASS_NOT_FOUND
	* @see #IL_OTHER
	*/
	public static int installLayout(String className) {
		int retVal = IL_OK;
		XelfiDesignLayout xdl;
		Debug.debug(Debug.LAYOUTS, "Installing layout: "+className+" ...");
		try {
			Class xx = Class.forName(className);
			xdl = (XelfiDesignLayout) xx.newInstance();
			layoutNames.addElement(xdl.getClassName());
			classNames.addElement(className);
			Debug.debug(Debug.LAYOUTS, "Layout: "+className+" succesfully installed");
		} catch (IllegalAccessException e) {
			retVal = IL_ILLEGAL_ACCESS;
		} catch (InstantiationException e) {
			retVal = IL_INSTANTIATION;
		} catch (ClassNotFoundException e) {
			retVal = IL_CLASS_NOT_FOUND;
		} catch (Exception e) {
			retVal = IL_OTHER;
		}		
		if (retVal == IL_OK) isDefault = false;
		return retVal;
	}

	/**
	* This method uninstalls given layout.
	* Given class name must be complete (i.e. with packages) name to design layout class 
	* (e.g. xelfi.design.forms.layouts.XFlowLayout).
	* If last layout is uninstalled, default set of layouts is installed.
	* @param className	complete name of design layout class to install
	* @see #installLayout
	*/
	public static void uninstallLayout(String className) {
		int i = classNames.indexOf(className);
		if (i!= -1) {
			classNames.removeElementAt(i);
			layoutNames.removeElementAt(i);
			isDefault = false;
		}
		if (classNames.size() == 0)
			layoutsDefault();
	}

	/**
	* This method creates new layout for given layout name.
	* Given name must be name of the layout itself, not the design layout class 
	* (e.g. FlowLayout, not xelfi.design.forms.layouts.XFlowLayout).
	* This method first searches installed layouts for given name and then dynamically creates new class
	* if the layout name is found.
	* This method is intended primarily for ComponentInspector.
	* @return created layout class or null if layout with such a name is not installed
	*/
	public static XelfiDesignLayout createLayout(String name) {
		String className = findClassName(name);
		XelfiDesignLayout xdl = null;
		if (className != null) {
			try {
				Class xx = Class.forName(className);
				 xdl = (XelfiDesignLayout) xx.newInstance();
			} catch (Exception e) {
			// [PENDING]
			}		
		}
		return xdl;
	}

	private static String findClassName(String layName) {
		int i = layoutNames.indexOf(layName);
		if (i==-1) return null;
		return (String)classNames.elementAt(i);
	}
	
	/**
	* Installs default set of layouts. 
	* This method is intended to be started at Xelfi startup when no layouts setting are stored
	* in options file.
	*/
	public static void layoutsDefault() {
		Debug.debug(Debug.LAYOUTS, "Installing default layouts...");
		String[] defaults = XFDefaultLayouts.get();
		for (int i=0; i<defaults.length; i++)
			installLayout(defaults[i]);
		isDefault = true;
		Debug.debug(Debug.LAYOUTS, "Default layouts succesfully installed.");
	}

	/**
	* Loads installed layouts set from properties 
	* (i.e. from .XelfiOptions file).
	* If there are no properties for layouts there, installs default set of layouts.
	* @param prop 	properties class
	* @see #storeLayouts
	* @see #layoutsDefault
	*/
	public static void loadLayouts(EnhProperties prop) {
		layoutNames.removeAllElements();
		classNames.removeAllElements();
		int count = prop.getIntProperty("INSTALLED_LAYOUTS_COUNT", -1);
		if (count == -1)	layoutsDefault();
		else
			for (int i = 0; i < count; i++)
				installLayout(prop.getProperty("INSTALLED_LAYOUT_"+i, null));
	}

	/**
	* Stores installed layouts set to properties 
	* (i.e. to .XelfiOptions file).
	* @param prop 	properties class
	* @see #loadLayouts
	*/
	public static void storeLayouts(EnhProperties prop) {
		if (isDefault) {
			prop.setIntProperty("INSTALLED_LAYOUTS_COUNT", -1);
			return;
		}
		prop.setIntProperty("INSTALLED_LAYOUTS_COUNT", classNames.size());
		for (int i = 0; i < classNames.size(); i++)
			prop.put("INSTALLED_LAYOUT_"+i,(String) classNames.elementAt(i));
	}
}
