/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.io.IOException;
import java.io.File;
/**
* Class that holds all opened documents and references to them. When
* all references disappears, the documents is removed.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class Editor extends Object {
  /** the source finder */
  private static SourceFinder sourceFinder = new StandardSourceFinder ();

  /** Hashtable with class names and LoadedDocuments */
  static Hashtable ref = new Hashtable ();

  /** Hashtable with modified sources. This forbids modified sources of
  * being released of memory. They will be only after the save.
  */
  static Hashtable modified = new Hashtable ();

  /** Loaders */
  static Vector loaders = new Vector ();


  /** How much spaces add when a load of TAB is encountered */
  public static int tabSize = 4;

  /** Create backups? */
  public static boolean backupFiles = true;

  /** preferred width */
  public static int preferredWidth = 80;
  /** preferred height */
  public static int preferredHeight = 25;

  /** To forbid creating of instances outside this class */
  private Editor () {
  };

  /** Removes reference to document.
  * @param c the name of the class
  * @number of left references
  */
  synchronized static void remove (String c) {
    ref.remove (c);
  };

  /** Opens new document. Ands sets it to the source object s.
  * @param xf the file to load from
  * @param s the source to assign document to
  * @param create create new class if it does not exist?
  * @return the document
  * @exception IOException if no source has been found
  */
  static synchronized void doStaticOpen (XelfiFile xf, Source s, boolean create) throws IOException {
    LoadedDocument d = (LoadedDocument)ref.get (xf.getPackageAndName ());
    if (d == null) {
      // we have to load new document
      d = new LoadedDocument (xf, create);
      s.doc = d;
      // this method can also throw a IOException
      notifyNewDocument (s);
      // now we can register it. Everything is all right
      ref.put (xf.getPackageAndName (), d);
    } else {
      s.doc = d;
    };
  };

  /** Attaches a new EditorLoader to the editor. This object will be
  * informed when a new file is loaded to the editor.
  * @param el the editor loader
  * @see EditorLoader
  */
  public static void addLoader (EditorLoader el) {
    loaders.addElement (el);
  };

  /** Removes a EditorLoader from the editor.
  * @param el the loader to remove
  */
  public static void removeLoader (EditorLoader el) {
    loaders.removeElement (el);
  };

  /** Notifies all loaders of loading a new document
  * @param s the source object for the opened document
  * @exception IOException an error during document notification
  */
  static synchronized void notifyNewDocument (Source s) throws IOException {
    Enumeration e = loaders.elements ();
    try {
      for (;;) {
        EditorLoader el = (EditorLoader)e.nextElement ();
        try {
          el.newDocumentHasBeenLoaded (s);
        } catch (IOException ex) {
          // we must inform all loaders before this one that the source has been released
          try {
            notifyOldDocument (s, el);
            // rethrow the exception
          } finally {
            s.doc = null;
          };
          throw ex;
        } catch (Exception ex) {
          // prints the exception
          ex.printStackTrace ();
        }
      }
    } catch (NoSuchElementException ex) {
    }
  }

  /** Notifies all loaders about document that is being closed.
  * @param s the source object for closed document
  * @warning the source object becomes invalid after this method
  */
  static synchronized void notifyOldDocument (Source s) {
    notifyOldDocument (s, null);
  };

  /** Notifies all loaders about document that is being closed.
  * @param s the source object for closed document
  * @param stop the loader that should not be informed
  * @warning the source object becomes invalid after this method
  */
  static void notifyOldDocument (Source s, EditorLoader stop) {
    Enumeration e = loaders.elements ();
    try {
      for (;;) {
        EditorLoader el = (EditorLoader)e.nextElement ();
        // stop in the specified loader
        if (el == stop) break;
        try {
          el.documentIsAboutToClose (s);
        } catch (Exception ex) {
          ex.printStackTrace ();
        };
      };
    } catch (NoSuchElementException ex) {
    };
  };
  
  /** Gets the enumeration of all opened documents. Elements can be cast
  * to Source.
  * @return the enumeration of sources of this object.
  * @see Source
  */
  public static Enumeration loadedSources () {
    return new SourceEnumeration (ref.elements ());
  };

  /** Gets the enumeration of all modified sources. Elements can be cast
  * to Source.
  * @return the enumeration of sources of this object.
  * @see Source
  */
  public static Enumeration modifiedSources () {
    return new SourceEnumeration (ref.elements (), true);
  };

  /** Saves all opened and modified files to disk.
  * @exception IOException if an error occures
  */
  public static void saveAll () throws IOException {
    try {
      Enumeration en = loadedSources ();
      for (;;) {
        ((Source)en.nextElement ()).save ();
      }
    } catch (NoSuchElementException ex) {
    }
    System.gc ();
    System.runFinalization ();
  }

  /** Gets a snapshot of all opened files. This is used to
  * maintain the link between line numbers and real lines
  * and preserve it under modification of source files.
  * @return the snapshot of all opened files
  */
  public static Snapshot getSnapshot () {
    return new Snapshot ();
  };

  /** Sets new source finder to the editor.
  * @param s the source finder
  */
  public static void setSourceFinder (SourceFinder s) {
    sourceFinder = s;
  };

  /** Tries to find source file for specified class. Uses editor's SourceFinder.
  * If the class does not exist new source file is created.
  * @param clazz the class name
  * @return the source file
  * @exception IOException if an error occures
  * @see SourceFinder
  * @see Editor#setSourceFinder 
  */
  public static XelfiFile findSource (String clazz) throws IOException {
    return findSource (clazz, true);
  };

  /** Tries to find source file for specified class. Uses editor's SourceFinder.
  * @param clazz the class name
  * @param create should the source be create if it does not exists
  * @return the source file
  * @exception IOException if an error occures
  * @see SourceFinder
  * @see Editor#setSourceFinder
  */
  public static XelfiFile findSource (String clazz, boolean create) throws IOException {
    return sourceFinder.findSource (clazz, create);
  };

  /** Tries to find source file for specified file in the file system. Uses editor's SourceFinder.
  * The file will not be created if it does not exist.
  * @param file the file
  * @return the source file
  * @exception IOException if an error occures
  * @see SourceFinder
  * @see Editor#setSourceFinder
  */
  public static XelfiFile findClass (File file) throws IOException {
    return sourceFinder.findClass (file);
  };
};







