/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
/**
* Extension to scrollbar class that is designed to work on a document,
* automaticly count number of lines.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class EditorVScrollbar extends Scrollbar implements ActionPostListener, EditorListener {
  /** the editor info */
  private TextEditorInfo editor;

  /** Number of lines in document. */
  private int lines;

  /** View port size */
  private int view;

  /** Indicates that the redrawing request comes from the scrollbar */
  private boolean redrawing = false;

  /** Constructs new scrollbar. Counts number of lines in document.
  * Adds itself as a controler to the document.
  * @param e the editor that this scrollbar belongs to
  */
  public EditorVScrollbar (EditorCanvas e) {
    super (VERTICAL);
    editor = e;
    editor.getDocument ().addActionPostListener (this);
    editor.addEditorListener (this);
    lines = editor.getDocument ().numberOfLines ();
    linesChanged ();
  };
  
  /** What to do on editor close.
  */
  public void close () {
    editor.getDocument ().removeActionPostListener (this);
    editor.removeEditorListener (this);
  };

  /** Sets correct values to the scrollbar.
  */
  public void linesChanged () {
    int v = 0;
    int l = lines - view;
    if (l < 0) l = 0;
    try {
      v = editor.getDocument ().lineToInt (editor.getVisibleBlock ().getBegin ());
    } catch (InvalidPositionException ex) {
    };
    if (v > l) v = l;
    setValues (v, 1, 0, l);
  };

  /** Handles scrollbar changes.
  * @param e the event
  */
  public boolean handleEvent (Event e) {
    try {
      redrawing = true;

      switch (e.id) {
      case Event.SCROLL_LINE_UP:
        editor.scrollUp ();
        break;
      case Event.SCROLL_LINE_DOWN:
        editor.scrollDown ();
        break;
      case Event.SCROLL_PAGE_UP:
        TextEditorOperation.pageUp (editor);
        break;
      case Event.SCROLL_PAGE_DOWN:
        TextEditorOperation.pageDown (editor);
        break;
      case Event.SCROLL_ABSOLUTE:
        Position p = editor.getDocument ().goToLine (getValue ());
        if (!p.beginOfLine ().equals (editor.getVisibleBlock ().getBegin ().beginOfLine ())) {
          editor.setVisibleBlockBegin (p);
          editor.setCursorPos (p);
        };
        break;
      default:
        return super.handleEvent(e);
      };
    } catch (EndOfTextException ex) {
    } catch (InvalidPositionException ex) {
    } finally {
      redrawing = false;
    };
    linesChanged ();
    return true;
  };

  /** Notifies that an insert has been done at position p.
  * @param pp the position where the insert occures (same as the position
  *   sent to allowInsert)
  * @param n the position of newly inserted character(s) (this is before p)
  */
  public void notifyInsert (Position n, Position pp) {
    String s = n.toString (pp);
    int p = s.indexOf ('\n');
    while (p >= 0) {
      lines++;
      p = s.indexOf ('\n', p + 1);
    };
    linesChanged ();
  };
  
  /** Notifies a delete
  * @param po the position where a delete occures
  * @param s the string with characters that has been deleted
  */
  public void notifyDelete (Position po, String s) {
    int p = s.indexOf ('\n');
    while (p >= 0) {
      lines--;
      p = s.indexOf ('\n', p + 1);
    };
    linesChanged ();
  };

  /** Called when action requested by user in editor is not allowed.
  * @param info info about document
  * @param ins true if insert, false if delete
  */
  public void editorNotAllowedAction (TextEditorInfo info)  {
  };
  
  /** When user double clicks with mouse this will be called.
  * @param info editor info
  * @param p position of click
  */
  public void editorDoubleClick (TextEditorInfo info, Position p) {
  };
  
  /** Notifies change of view (visible part of document)
  * @param info editor info
  */
  public void editorViewChanged (TextEditorInfo info) {
    if (!redrawing) {
      int v = 0;
      try {
        Position p = info.getVisibleBlock ().getBegin ();
        v = editor.getDocument ().lineToInt (p);
        int i = info.numberOfVisibleLines ();
        if (i != view) {
          view = i;
          linesChanged ();
        };
      } catch (InvalidPositionException ex) {
      };
      setValue (v);
    };
  };
  
};






