/**
* Xelfi programming environment
*/

package xelfi.editor;

import java.io.DataInput;
import java.io.IOException;
import java.io.EOFException;
/**
* This class represents one position in text editor.
* Positions are used to move around the document. It is posible to get
* previous, next, down and up position.
* The position is maintained always on the "same place". It means
* when somebody inserts a text before the position the position is moved.
* There are some basic operations on positions like insert, delete, find...
* Positions can be compared to each other.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public final class Position extends Object {
  /** constant to signals results of comparision of Positions */
  static final byte EQUAL = 0;
  static final byte LESS = -1;
  static final byte GREATER = 1;
  /** position are different when they are in different documents */
  static final byte DIFFERENT = -2;

  /** index on the line */
  Index index;

  /** creates new position
  * @param p the position implementation
  */
  Position (Index i) {
    line (index = i).lock (i);
  }

  /** Gets the line for this index.
  * @param i the index
  * @return the line
  */
  static Line line (Index i) {
    return i.getLine ();
  }

  /** Finalizer closes its implementation */
  public void finalize () {
    if (index != null) {
      Line l = line (index);
      if (l != null) l.unlock (index);
    }
  }

  /** Test if this is end of line.
  * @return true if it is
  */
  public final boolean isEndOfLine () {
    return line (index).isEndOfLine (index);
  }

  /** Test if this is begin of line.
  * @return true if it is
  */
  public final boolean isBeginOfLine () {
    return line (index).isBeginOfLine (index);
  }

  /** Test if this is last position on line.
  * @return true if it is
  */
  public final boolean isLastOfLine () {
    return line (index).isLastOfLine (index);
  }

  /** Is end of text?
  * @return true if this position is at end of document
  */
  public final boolean isEndOfText () {
    Line l = line (index);
    return l.nextLine == null && l.text.length () <= l.indexToInt (index);
  }

  /** Is begin of text?
  * @return true if it is
  */
  public final boolean isBeginOfText () {
    Line l = line (index);
    return l.prevLine == null && l.indexToInt (index) == 0;
  }


  /** Deletes text from this position till the second.
  * @param end the second position
  * @parma info the info
  * @warning the second position must follow the first
  */
  final synchronized void deleteTill (Position end, Object info) throws NotAllowedException {
    if (less (end)) {
      Document d = getDocument ();
      try {
        d.lock ();
        d.allowMultiDelete (this, end, info);

        String s = toString (end);

        try {
          d.beforeMultiDelete (this, end);
        } catch (Exception ex) {
          // if an error occures
          ex.printStackTrace ();
        }

        Line first = line (index);
        Line second = line (end.index);

        if (first == second) {
          // if both positions are on the same line
          first.deleteBetween (index, end.index);
        } else {
          first.deleteFrom (index);
          second.deleteTill (end.index);
          second = second.prevLine;
          while (second != first) {
            second = second.deleteLine ();
          }
          first.concatenateLine (index);
        }

        d.notifyDelete (this, s);

      } finally {
        d.unlock ();
      }
    }
  }


  /** Deletes one character.
  * @param info the info sent to allowDelete
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  */
  public final void delete (Object info) throws NotAllowedException {
    Document d = getDocument ();
    d.allowDelete (this, info);
    try {
      d.beforeDelete (this);
    } catch (Exception ex) {
      // if an error occures
      ex.printStackTrace ();
    }
    Line l = line (index);
    char ch = l.getChar (index);
    l.deleteChar (index);
    d.notifyDelete (this, String.valueOf (ch));
  }

  /** Deletes one character. The info is set to null.
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  */
  public final void delete () throws NotAllowedException {
    delete (null);
  }

  /** Inserts a string.
  * @param s string to insert
  * @param info the object that is passed
  * @return position of newly inserted character
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  * @see Document.allowInsert
  */
  public final Position insert (String s, Object info) throws NotAllowedException {
    Document d = getDocument ();
    d.allowInsert (this, info);
    try {
      d.beforeInsert (this, s);
    } catch (Exception ex) {
      // if an error occures
      ex.printStackTrace ();
    }
    Position p = new Position (line (index).insert (index, s));
    d.notifyInsert (p, this);
    return p;
  }

  /** Inserts a string. To allowInsert a null is sent.
  * @param s string to insert
  * @return position of newly inserted character
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  * @see Document.allowInsert
  */
  public final Position insert (String s) throws NotAllowedException {
    return insert (s, null);
  }

  /** Insert from a stream.
  * @param s the stream to insert from
  * @param info the info to be sent to allowInsert
  * @return position of newly inserted character
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  * @exception IOException if an error occures on read
  * @see Document.allowInsert
  */
  public final Position insert (DataInput s, Object info) throws NotAllowedException, IOException {
    Position beg = null;
    try {
      for (;;) {
        String str = s.readLine ();
        if (str == null) break;
        StringBuffer b = new StringBuffer (str);
        for (int i = 0; i < b.length (); i++) {
          char ch;
          if (((int)(ch = b.charAt (i))) < (int)' ') {
            b.setCharAt (i, ' ');
            if (ch == '\t') {
              do {
                b.insert (++i, ' ');
              } while ((i - 2)% Editor.tabSize != 0);
            }
          }
        }
        Position x = insert (b.toString (), info);
        if (beg == null) {
          beg = x;
        }
        insert ('\n', info);
      }
    } catch (EOFException e) {
    }
    return beg == null ? this : beg;
  }
  
  /** Insert from a stream. To allowInsert a null is sent.
  * @param s string to insert
  * @return position of newly inserted character
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  * @see Document.allowInsert
  */
  public final Position insert (DataInput s) throws NotAllowedException, IOException {
    return insert (s, null);
  }

  /** Inserts a character.
  * @param ch character to insert
  * @param info the object that is passed
  * @return position of newly inserted character
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  * @see Document.allowInsert
  */
  public final Position insert (char ch, Object info) throws NotAllowedException {
    return insert (String.valueOf (ch), info);
  }
  
  /** Inserts a character. To allowInsert a null is sent.
  * @param ch character to insert
  * @return position of newly inserted character
  * @exception NotAllowedException can be thrown if document do not allow
  *   this action. The method Document.allowInsert (Position) is called.
  * @see Document.allowInsert
  */
  public final Position insert (char ch) throws NotAllowedException {
    return insert (ch, null);
  }
  
  /** Finds position that is accepted.
  * @see Acceptor
  * @param ac returns first following position ac.accept (char) returns true
  * @param forward should we search forward or backward
  * @return resulting position
  * @exception EndOfTextException if end of text is reached
  */
  public final Position find (Acceptor ac, boolean forward) throws EndOfTextException {
    return find (ac, null, forward);
  }

  /** Finds position that is accepted.
  * @see Acceptor
  * @param ac returns first following position ac.accept (char) returns true
  * @param end the end position to stop search at
  * @param forward should we search forward or backward
  * @return resulting position
  * @exception EndOfTextException if end of text is reached
  */
  final Position find (Acceptor ac, Position end, boolean forward) throws EndOfTextException {
    IndexBase i = index;
    Line l = line (index);
    Position p;
    
    for (;;) {
      if (end != null && sameLine (end)) {
        return new Position (l.find (ac, i, end.index, forward));
      } else {
        try {
          return new Position (l.find (ac, i, forward));
        } catch (EndOfTextException ex) {
          // go to next line; if exists
          if (forward && ac.accept ('\n')) {
            return new Position (l.endOfLine ());
          }
          if ((l = forward ? l.nextLine : l.prevLine) == null) {
            throw ex;
          }
          if (!forward && ac.accept ('\n')) {
            return new Position (l.endOfLine ());
          }
          if (forward) {
            i = l;
          } else {
            p = new Position (l.endOfLine ());
            i = p.index;
          }
        }
      }
    }
  }

  /** finds matching position searches forward
  * @see Acceptor
  * @param ac returns first following position ac.accept (char) returns true
  * @return resulting position
  * @exception EndOfTextException if end of text is reached
  */
  public final Position find (Acceptor ac) throws EndOfTextException {
    return find (ac, true);
  }

  /** gets character
  * @return character at this position
  */
  public final char getChar () {
    return line (index).getChar (index);
  }


  /** finds end of line
  * @return position at end of line
  */
  public final Position endOfLine () {
    return new Position (line (index).endOfLine ());
  }

  /** Returns end of line or the last position on the line. Depends what is more to right.
  * @return the last used position on line
  */
  public final Position lastOfLine () {
    return new Position (line (index).lastOfLine ());
  }
  
  /**
  * @return column of this position (counts from 0)
  */
  public final int getColumn () {
    return line (index).indexToInt (index);
  }
  
  /**
  * @return the document that this position belongs to
  */
  public Document getDocument () {
    return line (index).getDocument ();
  }
  
  /** finds begin of line
  * @return positon at begininig of line
  */
  public final Position beginOfLine () {
    return new Position (line (index).beginOfLine ());
  }
  
  /** compares two positions
  * @param p second position
  * @return one of EQUAL, LESS, GREATER, DIFFERENT
  */
  final byte compare (Position p) {
    Line l1 = line (index);
    Line l2 = line (p.index);
    byte c = l1.compare (l2);
    if (c == EQUAL) {
      int i1 = l1.indexToInt (index);
      int i2 = l2.indexToInt (p.index);
      if (i1 == i2) return EQUAL;
      if (i1 < i2) return LESS;
      else return GREATER;
    } else {
      return c;
    }
  }
  
  /** 
  * @return next position 
  */
  public final Position next () {
    return next (1);
  }

  /** Next position after c characters.
  * @param n how much characters to right
  * @param i ignore end of line or stop at it
  * @return the n-th position to right
  */
  public final Position next (int c) {
    return new Position (line (index).indexToRight (index, (short)c));
  }

  /** Returns the position at specified column on the same line.
  * @param c the column
  * @return the position
  */
  public final Position column (int c) {
    Line l = line (index);
    return new Position (l.indexToRight (l, (short)c));
  }

  /**
  * @return previous position
  * @exception EndOfTextException if this is first position on line
  */
  public final Position previous () throws EndOfTextException {
    return previous (1);
  }

  /**
  * @param c how much characters to left
  * @return the n-th position to left
  * @exception EndOfTextException if begin of line is reached
  */
  public final Position previous (int c) throws EndOfTextException {
    return new Position (line (index).indexToLeft (index, (short)c));
  }

  /** Safe previous position.
  * @param c how much positions before return
  * @return the right position before or begin of line
  */
  public final Position safePrevious (int c) {
    try {
      return previous (c);
    } catch (EndOfTextException ex) {
      return new Position (line (index).beginOfLine ());
    }
  }
  /**
  * @return position one line up
  * @exception EndOfTextException if there is no such position
  */
  public final Position up () throws EndOfTextException {
    return up (1);
  }

  /**
  * @param c how much lines
  * @return position one line up
  * @exception EndOfTextException if there is no such position
  */
  public final Position up (int c) throws EndOfTextException {
    if (c < 0) throw new EndOfTextException ();
    Line l = line (index);
    int pos = l.indexToInt (index);
    while (c-- > 0) {
      l = l.prevLine;
      if (l == null) throw new EndOfTextException ();
    }
    return new Position (l.indexToRight (l.beginOfLine (), (short)pos));
  }

  /** Up n lines or at the begin of document.
  * @param n number of lines to move up
  * @return the position
  */
  public final Position safeUp (int c) {
    try {
      return up (c);
    } catch (EndOfTextException e) {
      return getDocument ().getBegin ();
    }
  }

  /**
  * @return position one line down
  * @exception EndOfTextException if there is no such one
  */
  public final Position down () throws EndOfTextException {
    return down (1);
  }

  /**
  * @param c how much lines
  * @return position one line down
  * @exception EndOfTextException if there is no such one
  */
  public final Position down (int c) throws EndOfTextException {
    if (c < 0) throw new EndOfTextException ();
    Line l = line (index);
    int pos = l.indexToInt (index);
    while (c-- > 0) {
      l = l.nextLine;
      if (l == null) throw new EndOfTextException ();
    }
    return new Position (l.indexToRight (l.beginOfLine (), (short)pos));
  }

  /** Down n lines or at the end of document.
  * @param n number of lines to move down
  * @return the position
  */
  public final Position safeDown (int c) {
    try {
      return down (c);
    } catch (EndOfTextException e) {
      return getDocument ().getEnd ();
    }
  }

  /** Returns position of next character. If it is on the end of line then
  * returns the begin of next one. Otherwise the next character.
  * @return the next character
  * @exception EndOfTextException if end of text is reached
  */
  public final Position nextChar () throws EndOfTextException {
    return isEndOfLine () ? down ().beginOfLine () : next ();
  }

  /** Returns position of previous character. If it is on the begin of line then
  * returns the end of previous one. Otherwise the prevoiou character.
  * @return the prev character
  * @exception EndOfTextException if end of text is reached
  */
  public final Position previousChar () throws EndOfTextException {
    try {
      return previous ();
    } catch (EndOfTextException ex) {
//      return up ().endOfLine ().previousChar ();
      return up ().endOfLine ();
    }
  }

  /** Tests if two positions are on the same line.
  * @param p the second position
  * @return true if they are on the same line
  */
  public final boolean sameLine (Position p) {
    return line (index).compare (line (p.index)) == EQUAL;
  }

  /**
  * @return true if this position is before p
  */
  public final boolean less (Position p) {
    return compare (p) == LESS;
  }
  
  /**
  * @return true if this position is after p
  */
  public final boolean greater (Position p) {
    return compare (p) == GREATER;
  }
  
  /**
  * @return true if this is after or at p
  */
  public final boolean greaterOrEqual (Position p) {
    byte b = compare (p);
    return b == GREATER || b == EQUAL;
  }
  
  /**
  * @return true if this is before or at p
  */
  public final boolean lessOrEqual (Position p) {
    byte b = compare (p);
    return b == LESS || b == EQUAL;
  }

  /** Takes the first position of two.
  * @param a the first one
  * @param b the second
  * @return the less
  */
  public static final Position min (Position a, Position b) {
    if (b.less (a)) {
      return b;
    } else {
      return a;
    }
  }

  /** Takes the greater position of the two specified.
  * @param a the first one
  * @param b the second
  * @return the less
  */
  public static final Position max (Position a, Position b) {
    if (b.greater (a)) {
      return b;
    } else {
      return a;
    }
  }

  /**
  * @return true if two position points to the same place in file
  */
  public final boolean equals (Object o) {
    if (o instanceof Position) {
      Line l1 = line (index);
      Line l2 = line (((Position)o).index);
      if (l1 != l2) return false;
      int i1 = l1.indexToInt (index);
      int i2 = l2.indexToInt (((Position)o).index);
      return i1 == i2;
    } else {
      return false;
    }
  }

  /** Returns index of line on witch this position is.
  * @return the line number
  */
  int lineNumber () {
    return line (index).lineNumber ();
  }

  /** Returns the input stream that contains everything from this position till the end of doc.
  * @return the stream
  */
  public DocumentInputStream getInputStream () {
    return new DocumentInputStream (this);
  }

  /** Converts two positions to a string. Assumes the second position is after this one.
  * @param sec the end of block
  */
  final String toString (Position sec) {
    Index beg = index;
    Index end = sec.index;
    Line first = Position.line (beg);
    Line second = Position.line (end);

    if (first == second) {
      // if both positions are on the same line
      return first.toString (beg, end);
    } else {
      StringBuffer sb = new StringBuffer ();
      String last = second.toStringTill (end);
      sb.append (first.toStringFrom (beg));
      first = first.nextLine;
      while (second != first) {
        sb.append ('\n');
        sb.append (first);
        first = first.nextLine;
      }
      sb.append ('\n');
      sb.append (last);
      return sb.toString ();
    }
  }

  /** Converts a position to string.
  * @return the visual representation of position
  */
  public String toString () {
    StringBuffer sb = new StringBuffer (Block.line (this).toString ());
    int len = sb.length ();
    int c = getColumn ();

    while (len++ < c) {
      sb.append (' ');
    }
    sb.insert (c, '^');

    sb.insert (0, " at ");
    sb.insert (0, c);

    return sb.toString ();
  }
}






