/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.Panel;
import java.awt.BorderLayout;

import java.awt.Frame;
import java.awt.Event;
import java.io.*;
/**
* @version 1.0
* @author Jaroslav Tulach
*/

public class TextEditor extends Panel implements TextEditorInfo {
  private EditorCanvas editor;
  private EditorVScrollbar vert;
  private EditorHScrollbar horz;
  private EditorCounter counter;

  /** Construct new text editor. Creates standard editor callback.
  * Does not use undo.
  *
  * @param d the document to display
  */
  public TextEditor (Document d) {
    this (d, false);
  };

  /** Construct new text editor. Creates standard editor callback.
  * @param d the document to display
  * @param u should the undo be enabled?
  */
  public TextEditor (Document d, boolean u) {
    setLayout (new BorderLayout ());
    editor = new EditorCanvas (d, u);
    editor.setEditorCallback (
      new StandardEditorCallback (this, new EditorInfo ())
    );

    add ("Center", editor);
    vert = new EditorVScrollbar (editor);
    add ("East", vert);

    Panel p = new Panel ();
    p.setLayout (new BorderLayout ());

    counter = new EditorCounter (editor);
    p.add ("West", counter);
    horz = new EditorHScrollbar (editor);
    p.add ("Center", horz);

    add ("South", p);
  }

  /** Sets new editor callback to the editor. The previous one
  * is returned.
  * @param c the new callback
  */
  public void setEditorCallback (EditorCallback c) {
    editor.setEditorCallback (c);
  };

  /** Only returns editor callback.
  * @return the editor callback
  */
  public EditorCallback getEditorCallback () {
    return editor.getEditorCallback ();
  };

  /** Finalizer. Calls close method for editor and vertical scrollbar.
  * Because both are registered in document access controler they deregisters
  * and that is why they can be garbage colected.
  */
  protected void finalize () {
    editor.close ();
    vert.close ();
    horz.close ();
    counter.close ();
  };

  /** Gets the document in this window.
  * @return the document
  */
  public final Document getDocument () {
    return editor.getDocument ();
  };

  /** Gets the current position in document
  * @return the position
  */
  public final Position getCursorPos () {
    return editor.getCursorPos ();
  };

  /** Sets the cursor. If it is out of screen then changes the screen view.
  * @param pos the position to set the cursor to
  */
  public final void setCursorPos (Position pos) {
    editor.setCursorPos (pos);
  };

  /** Scrolls up one line. Moves everything on the screen down and draws
  * the first line.
  * @exception EndOfTextException if begin of document reached
  */
  public void scrollUp () throws EndOfTextException {
    editor.scrollUp ();
  };
  
  /** Scrolls down one line. Moves everything on the screen up and draws
  * the bottom line.
  * @exception EndOfTextException if end of document reached
  */
  public void scrollDown () throws EndOfTextException {
    editor.scrollDown ();
  };

  /** Returns block selection. This block is still valid and only changes
  * its size. That is why you can get it once and that use the same object
  * till the end of work with editor.

  * @return the selected block
  */
  public final Block getSelectedBlock () {
    return editor.getSelectedBlock ();
  };

  /** Sets selected block to new value.
  * @param sel the block or null to clear the selection
  */
  public void setSelectedBlock (Block sel) {
    editor.setSelectedBlock (sel);
  };

  /** Gets visible block
  * @return the block that is visible in the editor
  */
  public Block getVisibleBlock () {
    return editor.getVisibleBlock ();
  };

  /** Sets visible block begin.
  * @param p position where the visible part should begin
  */
  public void setVisibleBlockBegin (Position p) {
    editor.setVisibleBlockBegin (p);
  };
  
  /** Get number of visible lines.
  * @return the number of lines that fits into editor window
  */
  public int numberOfVisibleLines () {
    return editor.numberOfVisibleLines ();
  };

  /** Begins text marking. This call should be followed by a call
  * to blockEdxtendEnd.
  */
  public void blockExtendBegin () {
    editor.blockExtendBegin ();
  };

  /** Ends marking. Must be called after startMark. Changes selected block.
  *
  */
  public void blockExtendEnd () {
    editor.blockExtendEnd ();
  };

  /** Returns editor hashtable. This should be used to store
  * information about the editor like special position,
  * find and replace dialog and so on...
  * <p>
  * Keys to the table should be strings and should start with the name
  * of class that operates on the hash table followed with @ and
  * identificator. Like:
  * <P>
  * <center>xelfi.editor.Editor@Position1</center>
  * @return the hash table
  */
  public java.util.Hashtable getHashtable () {
    return editor.getHashtable ();
  };

  /** Gets frame or dialog where this editor is stored.
  * @return the window where this editor is stored
  */
  public java.awt.Window getWindow () {
    return editor.getWindow ();
  };

  /** Adds new cursor listener.
  * @param l the listener
  * @see CursorListener
  */
  public void addCursorListener (CursorListener l) {
    editor.addCursorListener (l);
  };
  
  /** Adds new editor listener.
  * @param l the listener
  * @see EditorListener
  */
  public void addEditorListener (EditorListener l) {
    editor.addEditorListener (l);
  };
  
  /** Removes listener.
  * @param l the listener
  * @return true if l was in the listener list
  * @see CursorListener
  */
  public boolean removeCursorListener (CursorListener l) {
    return editor.removeCursorListener (l);
  };
  
  /** Removes listener.
  * @param l the listener
  * @return true if l was in the listener list
  * @see EditorListener
  */
  public boolean removeEditorListener (EditorListener l) {
    return editor.removeEditorListener (l);
  };

  /** Does one undo. Works only if undo has been allowed in constructor.
  * @return false if not allowed to do undo
  */
  public final boolean undo () {
//    return editor.undo ();
   return false;
  };

  /** Set focus to the editor canvas.
  */
  public boolean gotFocus (Event e, Object what) {
    editor.requestFocus ();
    return false;
  };

  /** Hashcode.
  * @return the unique hashcode that is same of all TextEditorInfo that references the same editor
  */
  public int hashCode () {
    return editor.hashCode ();
  };
  
  /** Equals.
  * @param o the second object
  * @return true if this and o references the same editor
  */
  public boolean equals (Object o) {
    if (o instanceof TextEditor) return this == o;
    if (o instanceof EditorCanvas) return editor == o;
    return false;
  };
};








