/** TopLevel class
* @author: Filip Dvorak
* @version: 1.41
*   improved by Ian on Mar,20 - method exitXelfi renamed to doExitXelfi and its first statement
*   commented out, new method exitXelfi added
*/

package xelfi.top;

import java.io.*;
import java.util.*;
import java.awt.*;

import xelfi.util.*;
import xelfi.Xelfi;
import xelfi.compiler.*;
import xelfi.execution.RunOptions;
import xelfi.top.visual.*;

import xelfi.design.inspector.*;
import xelfi.design.forms.*;
import xelfi.design.components.palette.*;
import xelfi.design.util.*;
import xelfi.top.toolbar.*;

import xelfi.editor.*;
import xelfi.xedit.*;
import xelfi.browser.view.*;
import xelfi.browser.document.*;
import xelfi.debugger.*;

  /** The class that keeps references to all xelfi objects. */
public class TopLevel
{
    /* Enumeration of some important top level windows:
    */
  static final int W_UNKNOWN = 0;
  static final int W_OUTPUT = 1;
  static final int W_EDITOR = 2;
  static final int W_MAINMENU = 3;
  static final int W_INSPECTOR = 4;
  static final int W_MENUEDITOR = 5;
  static final int W_WATCHES = 6;
  static final int W_LOCALS = 7;
  static final int W_CALLSTACK = 8;
  static final int W_THREADS = 9;
  static final int W_BREAKPOINTS = 10;
  static final int W_FORM_FRAME = 11;
  static final int W_FORM_DIALOG = 12;
  static final int W_FORM_APPLET = 13;
  static final int W_PALETTEWIN = 14;
  static final int W_BROWSER = 15;

  public static final int OS_WINDOWS95 = 1;
  public static final int OS_WINDOWSNT = 2;
  public static final int OS_SOLARIS = 3;
  public static final int OS_OS2 = 4;
    /** other system */
  public static final int OS_UNKNOWN = 5;

  public static final int JAVA_JDK_1_1 = 1;
    /** other java interpreter (but not jdk 1.1) */
  public static final int JAVA_UNKNOWN = 2;

  private static int osId, javaId;

      /* Vector of all exixting EditorFrame objects, managed by (un)registrate() meth. */
  private static Vector editors;
  private static Output output;
  private static XFComponentInspector inspector;
  private static XFMenuEditor menuEditor;
  private static Vector formFrames, formDialogs, formApplets, browsers;
  private static MainMenu menu;
  private static Debugger debugger;
  private static XFComponentPalette palette;
  private static XFToolbar toolbar;

    /** debug mode. Constants for switching mode - changes look of the window. Use in setDesktopMode method. */
  public static final int MODE_DEBUG = 1;
    /** normal mode. Constants for switching mode - changes look of the window. Use in setDesktopMode method. */
  public static final int MODE_NORMAL = 2;
  private static int deskMode;

    /* Currently focused top level window - assigned directly by TopWindowBase*/
  static TopWindowBase focused;
    /* Previous focused top level window - assigned directly by TopWindowBase*/
  static TopWindowBase lastFocused;

    /** One global Snapshot used by compiler and debugger. */
  private static Snapshot currentSnapshot;

    /** Vector of XelfiOptions interfaces */
  private static Vector optionRegister;

    /** Used by switchRealMode method to store last inspected form */
  private static XelfiDesignInspectable inspectableBackup;

  static
  {
    String s = System.getProperty("os.name", "unknown").toUpperCase();
    if((s.indexOf("WINDOWS") >= 0))  // Ms Windows
      if(s.indexOf("NT") >= 0) osId = OS_WINDOWSNT;
      else osId = OS_WINDOWS95;
    else if(s.indexOf("OS/2") >= 0) osId = OS_OS2;
    else if(s.indexOf("SOLARIS") >= 0) osId = OS_SOLARIS;
    else osId = OS_UNKNOWN;

    String j = System.getProperty("java.version", "unknown").toUpperCase();
    if(j.indexOf("1.1") >= 0) javaId = JAVA_JDK_1_1;
    else javaId = JAVA_UNKNOWN;
  }

    /** Initialization method. Is called from xelfi.Xelfi. Initializes all xelfi, creates all xelfi components.
      @param cmdlineOpt command line options */
  public static void init(Object cmdlineOpt[])
  {
    editors = new Vector(16);
    formFrames = new Vector(10);
    formDialogs = new Vector(10);
    formApplets = new Vector(10);
    browsers = new Vector(2);
    optionRegister = new Vector(6);

    EnhProperties ini = OptionsLoader.load((String)cmdlineOpt[Xelfi.CONFIG]);

      // desktop windows creation
    new MainMenu();

      // palette
    menu.add("Center", palette = new XFComponentPalette());
    ((CheckboxMenuItem)menu.getItem(MainMenu.miDesignDock)).setState(true);
    XFInstalledLayouts.layoutsDefault();

      // toolbar
    toolbar = new XFToolbar();
    menu.add("West", toolbar);
    toolbar.toolbarDefault();

    menu.setupMenuItems();

    resetSnapshot();

      // some xelfi options registration
    addOption(new XFColor());
    CompilerOptions.register();
    addOption(new RunOptions());
    addOption(new Desktop());

      // init of debugger, must be done before any JavaEditorFrame is created
    debugger = new Debugger();
    debugger.getWatches().getView().reshape(0, 0, 200, 100);
    debugger.getBreakpoints().getView().reshape(200, 0, 200, 100);
    debugger.getLocalsView().reshape(400, 0, 200, 100);
    debugger.getCallStackView().reshape(600, 0, 200, 100);
    debugger.getThreadsView().reshape(800, 0, 200, 100);

      // output window
    new Output();

      // init of the browser
    BrowserSettings.Init(menu);

      // init of compon. inspector
    new XFComponentInspector(menu);
    menuEditor = new XFMenuEditor();

    XelfiEditor.initialize();

    // dispatch options
      //palette
    if(ini == null || !palette.loadPalette(ini))
      palette.paletteDefault();
      // others
    if(ini != null)
    {
      Enumeration opts = optionRegister.elements();
      while(opts.hasMoreElements()) ((XelfiOptions)opts.nextElement()).loadProperties(ini);
    }

    Desktop.initDesktop();
  }

  static void exitXelfi()
  {
    boolean unsavedExist = false;
    for (Enumeration e = Editor.loadedSources (); e.hasMoreElements();) {
      Source s = (Source)e.nextElement ();
      if (s.isModified()) unsavedExist = true;
    }
    if (unsavedExist) {
      SaveAskDialog sad = new SaveAskDialog(menu);
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension windowSize = sad.size();
      sad.move((screenSize.width-windowSize.width)/2,(screenSize.height-windowSize.height)/2);
      sad.show();
    }
    else doExitXelfi();  // if there are no modified sources
  }

    /** Calling this method will close whole application. */
  static void doExitXelfi()
  {
    getDebugger().shutdown();

      // collect properties and store them
    EnhProperties p = new EnhProperties();
    getPalette().storePalette(p);

      // new way:
    Enumeration opts = optionRegister.elements();
    while(opts.hasMoreElements()) ((XelfiOptions)opts.nextElement()).storeProperties(p);
    OptionsLoader.store(p);  // saves p into file

    System.exit(0);
  }

    /** Get id if operating system.
      @return operating system, see symbolic constants in TopLevel
     */
  public static int getOS()
  {
    return osId;
  }

    /** Get id if java interpreter. Can be different under different OS
      @return java interpreter, symbolic constants exist in TopLevel
     */
  public static int getJavaVersion()
  {
    return javaId;
  }

    /** Registrate new XelfiOptions interface.
      @param o new options that will be used in the system */
  public static void addOption(XelfiOptions o)
  {
    optionRegister.addElement(o);
  }

  static Vector getOptions()
  {
    return (Vector)optionRegister.clone();
  }

  /* Registrates the top level window w, which should be descendent either of TopLevelFrame or
    TopLevelDialog.
   */
  static void register(TopWindowBase b)
  {
    Window w = b.getWindow();
    if(w instanceof JavaEditorFrame)
    {
      b.focusSwitching = true;
      b.id = W_EDITOR;
      editors.addElement(w);
    }
    else if(w instanceof Output)
    {
      output = (Output)w;
      b.focusSwitching = true;
      b.id = W_OUTPUT;
    }
    else if(w instanceof MainMenu)
    {
      menu = (MainMenu)w;
      b.focusSwitching = false;
      b.id = W_MAINMENU;
    }
   else if(w instanceof WatchesView)
    {
      b.focusSwitching = true;
      b.id = W_WATCHES;
    }
    else if(w instanceof LocalsView)
    {
      b.focusSwitching = true;
      b.id = W_LOCALS;
    }
    else if(w instanceof CallStackView)
    {
      b.focusSwitching = true;
      b.id = W_CALLSTACK;
    }
    else if(w instanceof ThreadsView)
    {
      b.focusSwitching = true;
      b.id = W_THREADS;
    }
    else if(w instanceof BreakpointsView)
    {
      b.focusSwitching = true;
      b.id = W_BREAKPOINTS;
    }
    else if(w instanceof XFMenuEditor)
    {
      menuEditor = (XFMenuEditor)w;
      b.focusSwitching = true;
      b.id = W_MENUEDITOR;
    }
    else if(w instanceof XFComponentInspector)
    {
      inspector = (XFComponentInspector)w;
      b.focusSwitching = true;
      b.id = W_INSPECTOR;
    }
    else if(w instanceof XFFormFrame)
    {
      formFrames.addElement(w);
      b.id = W_FORM_FRAME;
      b.focusSwitching = true;
    }
    else if(w instanceof XFFormDialog)
    {
      formDialogs.addElement(w);
      b.id = W_FORM_DIALOG;
      b.focusSwitching = true;
    }
    else if(w instanceof XFFormApplet)
    {
      formApplets.addElement(w);
      b.id = W_FORM_APPLET;
      b.focusSwitching = true;
    }
    else if(w instanceof PaletteWindow)
    {
      PaletteWindow pal = (PaletteWindow)w;
      b.id = W_PALETTEWIN;
      b.focusSwitching = false;
    }
    else if(w instanceof BrWindow)
    {
      browsers.addElement(w);
      b.id = W_BROWSER;
      b.focusSwitching = true;
    }
      // other cases
    else
    {
      b.id = W_UNKNOWN;
      b.focusSwitching = true;
    }
  }

  /* UnRegistrates the top level window w, which should be descendent either of TopLevelFrame or
    TopLevelDialog.
   */
  static void unregister(TopWindowBase b)
  {
    Window w = b.getWindow();
    switch(b.id)
    {
      case W_EDITOR:
        editors.removeElement(w);
        break;
      case W_OUTPUT:
        output = null;
        break;
      case W_INSPECTOR:
        inspector = null;
        break;
      case W_MAINMENU:
        menu = null;
        break;
      case W_MENUEDITOR:
        menuEditor = null;
        break;
      case W_FORM_FRAME:
        formFrames.removeElement(w);
        break;
      case W_FORM_DIALOG:
        formDialogs.removeElement(w);
        break;
      case W_FORM_APPLET:
        formApplets.removeElement(w);
        break;
      case W_BROWSER:
        browsers.removeElement(w);
        break;
    }
  }

    /** Called when corresponding menu item from the Design submenu was selected. */
  public static void switchRealMode(boolean toReal)
  {
    int newMode = (toReal) ? XFDesignConstants.DM_REAL : XFDesignConstants.DM_DESIGN;
    if(XFDesignOptions.designMode == newMode) return;
    XFDesignOptions.designMode = newMode;
    if(toReal)
    {
         inspectableBackup = inspector.getInspectable();
    inspector.inspectableChanged(null);
    }
    else
    {
      inspector.inspectableChanged(inspectableBackup);
    }

    for(int i = 0; i < formFrames.size(); i++)
    {
      Frame form = (Frame)formFrames.elementAt(i);
      if(newMode == XFDesignConstants.DM_REAL)
      {
        ((XelfiDesignForm)form).setSelectedComponents(null);
        if(inspector.getInspectable() == (XelfiDesignInspectable)form) inspector.selectionChanged();
      }
      if (form.isVisible())
      {
        form.hide(); form.show();
        form.invalidate(); form.validate();
      }
    }

    for(int i = 0; i < formDialogs.size(); i++)
    {
      Dialog form = (Dialog)formDialogs.elementAt(i);
      if (newMode == XFDesignConstants.DM_REAL)
      {
        ((XelfiDesignForm)form).setSelectedComponents(null);
        if (inspector.getInspectable() == (XelfiDesignInspectable)form) inspector.selectionChanged();
      }
      if (form.isVisible())
      {
        form.hide(); form.show();
        form.invalidate(); form.validate();
      }
    }

    for (int i = 0; i < formApplets.size(); i++)
    {
      Dialog form = (Dialog)formApplets.elementAt(i);
      if (newMode == XFDesignConstants.DM_REAL)
      {
        ((XelfiDesignForm)form).setSelectedComponents(null);
        if (inspector.getInspectable() == (XelfiDesignInspectable)form) inspector.selectionChanged();
      }
      if (form.isVisible())
      {
        form.hide(); form.show();
        form.invalidate(); form.validate();
      }
    }
  }

    /** Called when corresponding menu item from the Design submenu was selected. */
  public static void showGrid(boolean show)
  {
    if (XFDesignOptions.showGrid == show) return;
    XFDesignOptions.showGrid = show;
    if(XFDesignOptions.designMode == XFDesignConstants.DM_REAL) return;

    for(int i = 0; i < formFrames.size(); i++)
    {
      Frame form = (Frame)formFrames.elementAt(i);
      if (form.isVisible())
      { form.hide(); form.show(); }
    }

    for(int i = 0; i < formDialogs.size(); i++)
    {
      Dialog form = (Dialog)formDialogs.elementAt(i);
      if (form.isVisible())
      { form.hide(); form.show(); }
    }

    for(int i = 0; i < formApplets.size(); i++)
    {
      Dialog form = (Dialog)formApplets.elementAt(i);
      if (form.isVisible())
      { form.hide(); form.show(); }
    }
  }

  /**  Brings w to the front and conditionally gives it a focus.
    @param w the window
    @param focus if true the widow will be given the focus
  */
  public static void bringToTop(Window w, boolean focus)
  {
    if(!w.isVisible()) w.show();
      w.toFront();
    if(focus) w.requestFocus();
  }

  /** Get all opened editors
    @return Enumeration of JavaEditorFrame classes, which are all currently opened editor windows.
  */
  public static Enumeration allEditors()
  {
    return editors.elements();
  }

    /** Finds opened editor
      @param s source to find
      @return editor with the source or null if there's no such editor window */
  public static JavaEditorFrame findEditor(Source s)
  {
    JavaEditorFrame found = null;
    Enumeration allEds = allEditors();
    while(allEds.hasMoreElements())
      if((found = (JavaEditorFrame)allEds.nextElement()).getSource().equals(s)) return found;
    return null;
  }

    /** Finds opened editor
      @param className editor to find
      @return editor with the source of the class or null if there's no such editor window */
  public static JavaEditorFrame findEditor(String className)
  {
    JavaEditorFrame found = null;
    Enumeration allEds = allEditors();
    while(allEds.hasMoreElements())
      if((found = (JavaEditorFrame)allEds.nextElement()).getSource().getName().equals(className)) return found;
    return null;
  }

    /** Finds all opened editors for one class
      @param className editors to find
      @return elements of the Enumeration are JavaEditorFrame instances with the source for the class */
  public static Enumeration findEditors(String className)
  {
    Vector all = new Vector(4, 4);
    JavaEditorFrame found = null;
    Enumeration allEds = allEditors();
    while(allEds.hasMoreElements())
      if((found = (JavaEditorFrame)allEds.nextElement()).getSource().getName().equals(className)) all.addElement(found);
    return all.elements();
  }

  /** Show an editor window.
    This function finds editor window for the source (creates it if it doesn't exist) and brings it to the top,
    and conditionally gives it a focus.
    @param s source to find
    @param focus whether to give the focus to the editor
    @return editor with the source (either existing one or created one) */
  public static JavaEditorFrame showEditor(Source s, boolean focus)
  {
    JavaEditorFrame f = findEditor(s);
    if(f == null) f = new JavaEditorFrame(s);
    bringToTop(f, focus);
    return f;
  }

  /** Show an editor window.
    This function finds editor window for the class (creates it if it doesn't exist) and brings it to the top,
    and conditionally gives it a focus.
    @param className class source to search for
    @param focus whether to give the focus to the editor
    @return editor with the source (either existing one or created one) */
  public static JavaEditorFrame showEditor(String className, boolean focus)
  {
    Source s = null;
    try
    {
      s = new Source(className);    // OK. This constructor uses XelfiFile inside
    }
    catch(IOException e)
    {
      new MessageBox("Error", "Error in opening source file for " + className).show();
      return null;
    }
    return showEditor(s, focus);
  }

  /** Show an editor window.
    This function finds editor window for the class (creates it if it doesn't exist) and brings it to the top,
    and conditionally gives it a focus.
    @param file class source to search for
    @param focus whether to give the focus to the editor
    @return editor with the source (either existing one or created one) */
  public static JavaEditorFrame showEditor(XelfiFile file, boolean focus)
  {
    Source s = null;
    try
    {
      s = new Source(file);
    }
    catch(IOException e)
    {
      new MessageBox("Error", "Error in opening source file for " + file).show();
      return null;
    }
    return showEditor(s, focus);
  }

  /** Show an editor window on specified position.
    This function finds editor window for the class (creates it if it doesn't exist), brings it to the top,
    conditionally gives it a focus and scrolls it to to the proper position.
    @param s class source to search for
    @param p position to scroll the editor on, must be inside of the Source s
    @param focus whether to give the focus to the editor
    @return editor with the source (either existing one or created one) */
  public static JavaEditorFrame showEditor(Source s, Position p, boolean focus)
  {
    if(p.getDocument() != s.getDocument()) return null;
    JavaEditorFrame f = findEditor(s);
    if(f == null) f = new JavaEditorFrame(s);
    f.getTextEditor().setCursorPos(p);
    bringToTop(f, focus);
    return f;
  }

  /** Show an editor window on specified position.
    This function finds editor window for the class (creates it if it doesn't exist), brings it to the top,
    conditionally gives it a focus and scrolls it to to the proper position.
    @param className class source to search for
    @param p position to scroll the editor on, must be inside of the source of the class className
    @param focus whether to give the focus to the editor
    @return editor with the source (either existing one or created one) */
  public static JavaEditorFrame showEditor(String className, Position p, boolean focus)
  {
    Source s = null;
    try
    {
      s = new Source(className);
    }
    catch(IOException e)
    {
      new MessageBox("Error", "Error in opening source file for " + className).show();
      return null;
    }
    return showEditor(s, p, focus);
  }

  /** Show an editor window on specified position.
    This function finds editor window for the class (creates it if it doesn't exist), brings it to the top,
    conditionally gives it a focus and scrolls it to to the proper position.
    @param file class source to search for
    @param p position to scroll the editor on, must be inside of the source file
    @param focus whether to give the focus to the editor
    @return editor with the source (either existing one or created one) */
  public static JavaEditorFrame showEditor(XelfiFile file, Position p, boolean focus)
  {
    Source s = null;
    try
    {
      s = new Source(file);
    }
    catch(IOException e)
    {
      new MessageBox("Error", "Error in opening source file for " + file).show();
      return null;
    }
    return showEditor(s, p, focus);
  }

    /** Finds opened form frame form dialog or form applet window for source.
      @param src the source text for the form
      @return the form window or null if no such a window is opened. */
  public static XelfiDesignForm findForm(Source src)
  {
    int i, j = formFrames.size();
    XelfiDesignForm form = null;
    for(i = 0; i < j; i++)
    {
      form = (XelfiDesignForm)formFrames.elementAt(i);
      if(form.getSource().equals(src)) return form;
    }
    j = formDialogs.size();
    for(i = 0; i < j; i++)
    {
      form = (XelfiDesignForm)formDialogs.elementAt(i);
      if(form.getSource().equals(src)) return form;
    }
    j = formApplets.size();
    for(i = 0; i < j; i++)
    {
      form = (XelfiDesignForm)formApplets.elementAt(i);
      if(form.getSource().equals(src)) return form;
    }
    return null;
  }

    /** Finds opened form frame form dialog or form applet window.
      @param src the source text for the form
      @return the form window or null if no such a window is opened. */
  public static XelfiDesignForm findForm(XelfiFile src)
  {
    int i, j = formFrames.size();
    XelfiDesignForm form = null;
    String sourceClass = src.getPackageAndName();
    for(i = 0; i < j; i++)
    {
      form = (XelfiDesignForm)formFrames.elementAt(i);
      if(form.getSource().getName().equals(sourceClass)) return form;
    }
    j = formDialogs.size();
    for(i = 0; i < j; i++)
    {
      form = (XelfiDesignForm)formDialogs.elementAt(i);
      if(form.getSource().getName().equals(sourceClass)) return form;
    }
    j = formApplets.size();
    for(i = 0; i < j; i++)
    {
      form = (XelfiDesignForm)formApplets.elementAt(i);
      if(form.getSource().getName().equals(sourceClass)) return form;
    }
    return null;
  }

  /** Show a form window for specified class
    This function finds opened form window for the class (or creates it if it doesn't exist), brings it to the top,
    conditionally gives it a focus.
    @param xf class source for the form
    @param focus whether to give the focus to the editor
    @return the form (either existing one or created one) */
  public static XelfiDesignForm showForm(XelfiFile xf, boolean focus)
  {
    XelfiDesignForm form = findForm(xf);
    if(form == null) form = XFFormLoader.loadForm(xf);
    if(form != null) bringToTop((Window)form, focus);
    return form;
  }

  /** Show a form window for specified class
    This function finds opened form window for the class (or creates it if it doesn't exist), brings it to the top,
    conditionally gives it a focus.
    @param src class source for the form
    @param focus whether to give the focus to the editor
    @return the form (either existing one or created one) */
  public static XelfiDesignForm showForm(Source src, boolean focus)
  {
    XelfiDesignForm form = findForm(src);
    if(form == null) form = XFFormLoader.loadForm(src.getFile());
    if(form != null) bringToTop((Window)form, focus);
    return form;
  }

  /** Returns currently focused Xelfi top level window or null if no window is focused.
   */
  public static Window currentlyFocused()
  {
    if(focused == null) return null;
    else return focused.getWindow();
  }

  /** Returns currently focused Xelfi top level window or last focused window if currently
    focused is null. This is better function to obtain Xelfi focused window.
   */
  public static Window getFocused()
  {
    return focused == null ?
      (lastFocused == null ? null : lastFocused.getWindow())  : focused.getWindow();
  }

  static TopWindowBase getFocusedBase()
  {
    return focused == null ?
      (lastFocused == null ? null : lastFocused)  : focused;
  }

  /** Returns focused Xelfi editor window or null if no editor window is focused.
   */
  public static JavaEditorFrame focusedEditor()
  {
    Window ed = getFocused();
    if(ed instanceof JavaEditorFrame) return (JavaEditorFrame)ed;
    else return null;
  }

  /** Get focused classes.
    First checks if editor or form is focused window and return it's class name. If this fails asks the
    ClassBrowser for a selected classes.
    @return names of the classes - elements of the Vector are instances of the String
  */
  public static Vector getFocusedClasses()
  {
    Vector v = new Vector(4);
    TopWindowBase b = getFocusedBase();
    if(b != null && b.id == W_EDITOR) v.addElement(((JavaEditorFrame)b.getWindow()).getSource().getName());
    else if(b != null && (b.id == W_FORM_FRAME || b.id == W_FORM_DIALOG || b.id == W_FORM_APPLET))
      v.addElement(((XelfiDesignForm)b.getWindow()).getSource().getName());
    else
    {    // check ClassBrowser
      BrObject selected[] = ComponentManager.getSelectedObject();
      for(int i = 0; i < selected.length; i++)
        v.addElement(selected[i].getXFile().getPackageAndName());
    }
    return v;
  }

  /** The same as getFocusedClasses except that instead of all classes it returns only one (it chooses the first one).
    If no class is focused it returns null.*/
  public static String getFocusedClass()
  {
    try
    {
      return (String)getFocusedClasses().firstElement();
    }
    catch(NoSuchElementException e) { return null; }
  }

  public static Output getOutput()
  {
    return output;
  }

  public static MainMenu getMenu()
  {
    return menu;
  }

  public static XFToolbar getToolbar()
  {
    return toolbar;
  }

  public static Debugger getDebugger()
  {
    return debugger;
  }

  public static XelfiComponentPalette getPalette()
  {
    return palette;
  }

  public static XelfiComponentInspector getInspector()
  {
    return inspector;
  }

  public static XelfiMenuEditor getMenuEditor()
  {
    return menuEditor;
  }

  public static Vector getBrowsers()
  {
    return browsers;
  }

  public static Vector getFormFrames()
  {
    return formFrames;
  }

  public static Vector getFormDialogs()
  {
    return formDialogs;
  }

  public static Vector getFormApplets()
  {
    return formApplets;
  }

  public static Snapshot getCurrentSnapshot()
  {
    return currentSnapshot;
  }

    // change snapshot befor copilation only !
  public static void setCurrentSnapshot(Snapshot sn)
  {
    Snapshot oldSnapshot = currentSnapshot;
    currentSnapshot = sn;
    debugger.currentSnapshotHasChanged(oldSnapshot);
  }

    // change snapshot befor copilation only !
  public static void resetSnapshot()
  {
    Snapshot oldSnapshot = currentSnapshot;
    currentSnapshot = Editor.getSnapshot();
    if(debugger != null) debugger.currentSnapshotHasChanged(oldSnapshot);
  }

  public static void setDesktopMode(int modeType)
  {
    deskMode = modeType;
    if(modeType == MODE_DEBUG) toolbar.toolbarDefault(XFToolbarDefault.TOOLBAR_DEBUGGING);
    else toolbar.toolbarDefault(XFToolbarDefault.TOOLBAR_NORMAL);
  }

  public static int getDesktopMode()
  {
    return deskMode;
  }
}
