/**
* Xelfi programming environment
*/

package xelfi.xedit;
import xelfi.templates.TemplateManager;
import xelfi.templates.PlainManager;
import xelfi.templates.Template;

import xelfi.editor.Source;
import xelfi.editor.Document;
import xelfi.editor.Block;
import xelfi.editor.Position;
import xelfi.editor.EndOfTextException;
import xelfi.editor.DocumentHandler;

import java.util.NoSuchElementException;

import java.awt.Event;

import java.io.File;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
/**
* Template manager for files that at some positions
* contain names of package and of the class.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class SubstManager extends PlainManager {
  /** Info about blocks in the memory
  */
  String blocks;

  /** It also must have default contructor. Otherwise the
  * method newInstance does not work.
  */
  public SubstManager () {
  };

  /** Creates new template manager and
  * sets a block info to it.
  * @param b the block info
  */
  private SubstManager (String b) {
    blocks = b;
  };


  /** How to find a subst controler for a source?
  * Use SEARCH event.
  * @param s the source
  */
  private SubstController getController (Source s) {
    DocumentHandler d = s.getHandler ();
    return (SubstController)d;
    // should not throw cast exception but one never knows
  };

  /** Does not store anything, because the PlainManager
  * has no special parameters.
  * @param t the template
  * @return the string to add to the specification of template
  */
  public String storeInfo (Template t) {
    return blocks;
  };

  /** Does not load anything.
  * @param t the template to store information to
  * @param s the string to read parameters from
  * @exception IllegalArgumentException if the parameters in s have wrong format
  */
  public void loadInfo (Template t, String s) throws IllegalArgumentException {
    blocks = s;
  };

  /** Copies the file for template t to the source.
  *
  * @param t the template for creating
  * @param s the target source
  * @exception Exception if something went wrong
  */
  public void newSource (Template t, Source s) throws Exception {
    File file = fileForTemplate (t);

    DataInputStream is = new DataInputStream (
      new FileInputStream (file)
    );
    Document d = s.getDocument ();
    d.delete ();
    d.getBegin ().insert (is);
    SubstController sc = new SubstController (blocks, s);
    sc.generate (s);

    s.setHandler (sc);
  };

  static final String signatureHeader = "// Generated from substitution template \'";
  static final String signatureEnd = "\'";
  static final String blocksBegin = "Blocks:";

  /** This method reads header in the last line and tries
  * to parse the default header.
  * @param s the source
  */
  public Template identifySource (Source s) throws NoSuchElementException {
    // header is on the last non empty line
    Position p = s.getDocument ().getEnd ().beginOfLine ();
    try {
      while (p.isEndOfLine ()) {
        p = p.up ().beginOfLine ();
      };
    } catch (EndOfTextException ex) {
      // begin of text reached without finding non empty line
      throw new NoSuchElementException ();
    };

    Block b = Block.line (p);
    // if not recognized then NoSuch.. is thrown here
    String line = b.toString ();
    Template t = lookForSignature (line, signatureHeader, signatureEnd);
    // delete the signature if found
    try {
      b.delete ();
    } catch (xelfi.editor.NotAllowedException ex) {
    };

    try {
      int bl = line.indexOf (blocksBegin);
      if (bl >= 0) {
        bl += blocksBegin.length ();
        String rest = line.substring (bl);
        SubstController sc = new SubstController (rest, s);
        s.setHandler (sc);
      };
    } catch (Exception ex) {
      ex.printStackTrace ();
    };

    return t;
  };

  /** The contents of source s is copied to the
  * template directory.
  *
  * @param old the old template
  * @param s the source
  * @param theNew the new template that should be created
  * @return returns template manager for this template (can be this or something else)
  * @exception Exception if the creating is not possible
  */
  public TemplateManager createTemplate (Template old, Source s, Template theNew) throws Exception {
    File file = fileForTemplate (theNew);
    DataOutputStream os = new DataOutputStream (
      new FileOutputStream (file)
    );
    s.getDocument ().save (os);
    os.close ();
    SubstController sc = getController (s);
    String sav = sc.save ();
    return new SubstManager (sav);
  };

};










